@@ -28,6 +28,7 @@ import org.apache.karaf.admin.Instance;
 import org.apache.karaf.jpm.Process;
 import org.apache.karaf.jpm.ProcessBuilderFactory;
 import org.apache.karaf.jpm.impl.ScriptUtils;
+import org.fusesource.jansi.Ansi;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -52,17 +53,19 @@ public class InstanceImpl implements Instance {
     private AdminServiceImpl service;
     private String name;
     private String location;
+    private String javaOpts;
     private Process process;
     private boolean root;
 
-    public InstanceImpl(AdminServiceImpl service, String name, String location) {
-        this(service, name, location, false);
+    public InstanceImpl(AdminServiceImpl service, String name, String location, String javaOpts) {
+        this(service, name, location, javaOpts, false);
     }
     
-    public InstanceImpl(AdminServiceImpl service, String name, String location, boolean root) {
+    public InstanceImpl(AdminServiceImpl service, String name, String location, String javaOpts, boolean root) {
         this.service = service;
         this.name = name;
         this.location = location;
+        this.javaOpts = javaOpts;
         this.root = root;
     }
 
@@ -77,6 +80,10 @@ public class InstanceImpl implements Instance {
     public String getName() {
         return this.name;
     }
+
+    public void setName(String name) {
+        this.name = name;
+    }
     
     public boolean isRoot() {
         return root;
@@ -86,6 +93,10 @@ public class InstanceImpl implements Instance {
         return location;
     }
 
+    public void setLocation(String location) {
+        this.location = location;
+    }
+
     public boolean exists() {
         return new File(location).isDirectory();
     }
@@ -95,7 +106,7 @@ public class InstanceImpl implements Instance {
         return this.process != null ? this.process.getPid() : 0;
     }
 
-    public int getPort() {
+    public int getSshPort() {
         InputStream is = null;
         try {
             File f = new File(location, "etc/org.apache.karaf.shell.cfg");
@@ -117,7 +128,7 @@ public class InstanceImpl implements Instance {
         }
     }
 
-    public void changePort(int port) throws Exception {
+    public void changeSshPort(int port) throws Exception {
         checkProcess();
         if (this.process != null) {
             throw new IllegalStateException("Instance not stopped");
@@ -139,16 +150,114 @@ public class InstanceImpl implements Instance {
         }
     }
 
+    public int getRmiRegistryPort() {
+        InputStream is = null;
+        try {
+            File f = new File(location, "etc/org.apache.karaf.management.cfg");
+            is = new FileInputStream(f);
+            Properties props = new Properties();
+            props.load(is);
+            String loc = props.getProperty("rmiRegistryPort");
+            return Integer.parseInt(loc);
+        } catch (Exception e) {
+            return 0;
+        } finally {
+            if (is != null) {
+                try {
+                    is.close();
+                } catch (IOException e) {
+                    // Ignore
+                }
+            }
+        }
+    }
+
+    public void changeRmiRegistryPort(int port) throws Exception {
+        checkProcess();
+        if (this.process != null) {
+            throw new IllegalStateException("Instance not stopped");
+        }
+        Properties props = new Properties();
+        File f = new File(location, "etc/org.apache.karaf.management.cfg");
+        InputStream is = new FileInputStream(f);
+        try {
+            props.load(is);
+        } finally {
+            is.close();
+        }
+        props.setProperty("rmiRegistryPort", Integer.toString(port));
+        OutputStream os = new FileOutputStream(f);
+        try {
+            props.store(os, null);
+        } finally {
+            os.close();
+        }
+    }
+
+    public int getRmiServerPort() {
+        InputStream is = null;
+        try {
+            File f = new File(location, "etc/org.apache.karaf.management.cfg");
+            is = new FileInputStream(f);
+            Properties props = new Properties();
+            props.load(is);
+            String loc = props.getProperty("rmiServerPort");
+            return Integer.parseInt(loc);
+        } catch (Exception e) {
+            return 0;
+        } finally {
+            if (is != null) {
+                try {
+                    is.close();
+                } catch (IOException e) {
+                    // Ignore
+                }
+            }
+        }
+    }
+
+    public void changeRmiServerPort(int port) throws Exception {
+        checkProcess();
+        if (this.process != null) {
+            throw new IllegalStateException("Instance not stopped");
+        }
+        Properties props = new Properties();
+        File f = new File(location, "etc/org.apache.karaf.management.cfg");
+        InputStream is = new FileInputStream(f);
+        try {
+            props.load(is);
+        } finally {
+            is.close();
+        }
+        props.setProperty("rmiServerPort", Integer.toString(port));
+        OutputStream os = new FileOutputStream(f);
+        try {
+            props.store(os, null);
+        } finally {
+            os.close();
+        }
+    }
+
+    public String getJavaOpts() {
+        return javaOpts;
+    }
+
+    public void changeJavaOpts(String javaOpts) throws Exception {
+        this.javaOpts = javaOpts;
+        this.service.saveState();
+    }
+
     public synchronized void start(String javaOpts) throws Exception {
         checkProcess();
         if (this.process != null) {
             throw new IllegalStateException("Instance already started");
         }
-        if (javaOpts == null) {
+        if (javaOpts == null || javaOpts.length() == 0) {
+            javaOpts = this.javaOpts;
+        }
+        if (javaOpts == null || javaOpts.length() == 0) {
             javaOpts = "-server -Xmx512M -Dcom.sun.management.jmxremote";
         }
-        String karafOpts = System.getProperty("karaf.opts", "");  
-        
         File libDir = new File(System.getProperty("karaf.home"), "lib");
         File[] jars = libDir.listFiles(new FilenameFilter() {
             public boolean accept(File dir, String name) {
@@ -163,8 +272,7 @@ public class InstanceImpl implements Instance {
             classpath.append(jar.getCanonicalPath());
         }
         String command = new File(System.getProperty("java.home"), ScriptUtils.isWindows() ? "bin\\java.exe" : "bin/java").getCanonicalPath()
-                + " " + javaOpts 
-                + " " + karafOpts
+                + " " + javaOpts
                 + " -Djava.util.logging.config.file=\"" + new File(location, "etc/java.util.logging.properties").getCanonicalPath() + "\""
                 + " -Djava.endorsed.dirs=\"" + new File(new File(new File(System.getProperty("java.home"), "jre"), "lib"), "endorsed") + System.getProperty("path.separator") + new File(new File(System.getProperty("java.home"), "lib"), "endorsed") + System.getProperty("path.separator") + new File(libDir, "endorsed").getCanonicalPath() + "\""
                 + " -Djava.ext.dirs=\"" + new File(new File(new File(System.getProperty("java.home"), "jre"), "lib"), "ext") + System.getProperty("path.separator") + new File(new File(System.getProperty("java.home"), "lib"), "ext") + System.getProperty("path.separator") + new File(libDir, "ext").getCanonicalPath() + "\""
@@ -204,9 +312,8 @@ public class InstanceImpl implements Instance {
         this.service.saveState();
     }
 
-
     public synchronized String getState() {
-        int port = getPort();
+        int port = getSshPort();
         if (!exists() || port <= 0) {
             return ERROR;
         }
