@@ -28,6 +28,10 @@ public interface Feature {
 
     String getName();
 
+    String getDescription();
+
+    String getDetails();
+
     String getVersion();
 
     String getResolver();
@@ -38,4 +42,5 @@ public interface Feature {
 
     Map<String, Map<String, String>> getConfigurations();
 
+    List<ConfigFileInfo> getConfigurationFiles();
 }
