@@ -16,6 +16,8 @@
  */
 package org.apache.karaf.features.internal;
 
+import static java.lang.String.format;
+
 import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.FileInputStream;
@@ -23,29 +25,56 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.net.MalformedURLException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.Dictionary;
+import java.util.EnumSet;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Hashtable;
+import java.util.Iterator;
+import java.util.LinkedHashSet;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.Properties;
+import java.util.Set;
+import java.util.TreeSet;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.jar.JarInputStream;
 import java.util.jar.Manifest;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import org.apache.felix.utils.manifest.Clause;
+import org.apache.felix.utils.manifest.Parser;
+import org.apache.felix.utils.version.VersionRange;
+import org.apache.felix.utils.version.VersionTable;
 import org.apache.karaf.features.BundleInfo;
+import org.apache.karaf.features.ConfigFileInfo;
 import org.apache.karaf.features.Feature;
 import org.apache.karaf.features.FeatureEvent;
 import org.apache.karaf.features.FeaturesListener;
 import org.apache.karaf.features.FeaturesService;
 import org.apache.karaf.features.Repository;
 import org.apache.karaf.features.RepositoryEvent;
-import org.apache.felix.utils.manifest.Clause;
-import org.apache.felix.utils.manifest.Parser;
-import org.apache.felix.utils.version.VersionRange;
-import org.apache.felix.utils.version.VersionTable;
-import org.apache.karaf.features.*;
-import org.osgi.framework.*;
+import org.apache.karaf.features.Resolver;
+import org.osgi.framework.Bundle;
+import org.osgi.framework.BundleContext;
+import org.osgi.framework.BundleException;
+import org.osgi.framework.Constants;
+import org.osgi.framework.FrameworkEvent;
+import org.osgi.framework.FrameworkListener;
+import org.osgi.framework.FrameworkUtil;
+import org.osgi.framework.InvalidSyntaxException;
+import org.osgi.framework.Version;
 import org.osgi.service.cm.Configuration;
 import org.osgi.service.cm.ConfigurationAdmin;
 import org.osgi.service.packageadmin.PackageAdmin;
@@ -54,17 +83,6 @@ import org.osgi.util.tracker.ServiceTracker;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.io.*;
-import java.net.URI;
-import java.net.URISyntaxException;
-import java.net.URL;
-import java.util.*;
-import java.util.concurrent.CopyOnWriteArrayList;
-import java.util.jar.JarInputStream;
-import java.util.jar.Manifest;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
-
 import static java.lang.String.format;
 
 /**
@@ -174,6 +192,13 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
         this.boot = boot;
     }
 
+    /**
+     * Validate repository.
+     */
+    public void validateRepository(URI uri) throws Exception {
+        FeatureValidationUtil.validate(uri);
+    }
+
     public void addRepository(URI uri) throws Exception {
         if (!repositories.containsKey(uri)) {
             internalAddRepository(uri);
@@ -182,8 +207,12 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
     }
 
     protected RepositoryImpl internalAddRepository(URI uri) throws Exception {
-    	RepositoryImpl repo = null;
+        validateRepository(uri);
+        RepositoryImpl repo = null;
         repo = new RepositoryImpl(uri);
+        if (repo.getName() == null) {
+            LOGGER.warn("Feature repository doesn't have a name. The name will be mandatory in the next Karaf version.");
+        }
         repositories.put(uri, repo);
         repo.load();
         callListeners(new RepositoryEvent(repo, RepositoryEvent.EventType.RepositoryAdded, false));
@@ -257,12 +286,13 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
     public void installFeatures(Set<Feature> features, EnumSet<Option> options) throws Exception {
         InstallationState state = new InstallationState();
         InstallationState failure = new InstallationState();
+        boolean verbose = options.contains(FeaturesService.Option.Verbose);
         try {
             // Install everything
             for (Feature f : features) {
                 InstallationState s = new InstallationState();
             	try {
-                    doInstallFeature(s, f);
+                    doInstallFeature(s, f, verbose);
                     state.bundleInfos.putAll(s.bundleInfos);
                     state.bundles.addAll(s.bundles);
                     state.features.putAll(s.features);
@@ -272,7 +302,7 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
                     failure.features.putAll(s.features);
                     failure.installed.addAll(s.installed);
                     if (options.contains(Option.ContinueBatchOnFailure)) {
-                        LOGGER.info("Error when installing feature {}: {}", f.getName(), e);
+                        LOGGER.warn("Error when installing feature {}: {}", f.getName(), e);
                     } else {
                         throw e;
                     }
@@ -387,14 +417,43 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
         final Map<Feature, Set<Long>> features = new HashMap<Feature, Set<Long>>();
     }
 
-    protected void doInstallFeature(InstallationState state, Feature feature) throws Exception {
+    protected void doInstallFeature(InstallationState state, Feature feature, boolean verbose) throws Exception {
+        LOGGER.info("Installing feature " + feature.getName() + " " + feature.getVersion());
+        if (verbose) {
+            System.out.println("Installing feature " + feature.getName() + " " + feature.getVersion());
+        }
         for (Feature dependency : feature.getDependencies()) {
-            Feature f = getFeature(dependency.getName(), dependency.getVersion());
-            if (f == null) {
+            VersionRange range = FeatureImpl.DEFAULT_VERSION.equals(dependency.getVersion())
+                        ? VersionRange.ANY_VERSION : new VersionRange(dependency.getVersion(), true, true);
+            Feature fi = null;
+            for (Feature f : installed.keySet()) {
+                if (f.getName().equals(dependency.getName())) {
+                    Version v = VersionTable.getVersion(f.getVersion());
+                    if (range.contains(v)) {
+                        if (fi == null || VersionTable.getVersion(fi.getVersion()).compareTo(v) < 0) {
+                            fi = f;
+                        }
+                    }
+                }
+            }
+            if (fi == null) {
+                Map<String, Feature> avail = getFeatures().get(dependency.getName());
+                if (avail != null) {
+                    for (Feature f : avail.values()) {
+                        Version v = VersionTable.getVersion(f.getVersion());
+                        if (range.contains(v)) {
+                            if (fi == null || VersionTable.getVersion(fi.getVersion()).compareTo(v) < 0) {
+                                fi = f;
+                            }
+                        }
+                    }
+                }
+            }
+            if (fi == null) {
                 throw new Exception("No feature named '" + dependency.getName()
                         + "' with version '" + dependency.getVersion() + "' available");
             }
-        	doInstallFeature(state, f);
+            doInstallFeature(state, fi, verbose);
         }
         for (String config : feature.getConfigurations().keySet()) {
             Dictionary<String,String> props = new Hashtable<String, String>(feature.getConfigurations().get(config));
@@ -402,7 +461,7 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
             Configuration cfg = findExistingConfiguration(configAdmin, pid[0], pid[1]);
             if (cfg == null) {
                 cfg = createConfiguration(configAdmin, pid[0], pid[1]);
-                String key = (pid[1] == null ? pid[0] : pid[0] + "-" + pid[1]);
+                String key = createConfigurationKey(pid[0], pid[1]);
                 props.put(CONFIG_KEY, key);
                 if (cfg.getBundleLocation() != null) {
                     cfg.setBundleLocation(null);
@@ -410,28 +469,54 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
                 cfg.update(props);
             }
         }
+        for (ConfigFileInfo configFile : feature.getConfigurationFiles()) {
+        	installConfigurationFile(configFile.getLocation(), 
+        			configFile.getFinalname(), configFile.isOverride(), verbose);
+        }
         Set<Long> bundles = new TreeSet<Long>();
         for (BundleInfo bInfo : resolve(feature)) {
-            Bundle b = installBundleIfNeeded(state, bInfo);
+            Bundle b = installBundleIfNeeded(state, bInfo, verbose);
             bundles.add(b.getBundleId());
             state.bundleInfos.put(b.getBundleId(), bInfo);
         }
         state.features.put(feature, bundles);
     }
 
+    private String createConfigurationKey(String pid, String factoryPid) {
+        return factoryPid == null ? pid : pid + "-" + factoryPid;
+    }
+
     protected List<BundleInfo> resolve(Feature feature) throws Exception {
         String resolver = feature.getResolver();
         // If no resolver is specified, we expect a list of uris
         if (resolver == null || resolver.length() == 0) {
         	return feature.getBundles();
         }
+        boolean optional = false;
+        if (resolver.startsWith("(") && resolver.endsWith(")")) {
+            resolver = resolver.substring(1, resolver.length() - 1);
+            optional = true;
+        }
         // Else, find the resolver
         String filter = "(&(" + Constants.OBJECTCLASS + "=" + Resolver.class.getName() + ")(name=" + resolver + "))";
         ServiceTracker tracker = new ServiceTracker(bundleContext, FrameworkUtil.createFilter(filter), null);
         tracker.open();
         try {
-            Resolver r = (Resolver) tracker.waitForService(resolverTimeout);
-            return r.resolve(feature);
+            if (optional) {
+                Resolver r = (Resolver) tracker.getService();
+                if (r != null) {
+                    return r.resolve(feature);
+                } else {
+                    LOGGER.debug("Optional resolver '" + resolver + "' not found, using the default resolver");
+                    return feature.getBundles();
+                }
+            } else {
+                Resolver r = (Resolver) tracker.waitForService(resolverTimeout);
+                if (r == null) {
+                    throw new Exception("Unable to find required resolver '" + resolver + "'");
+                }
+                return r.resolve(feature);
+            }
         } finally {
             tracker.close();
         }
@@ -557,7 +642,7 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
         return result;
     }
 
-    protected Bundle installBundleIfNeeded(InstallationState state, BundleInfo bundleInfo) throws IOException, BundleException {
+    protected Bundle installBundleIfNeeded(InstallationState state, BundleInfo bundleInfo, boolean verbose) throws IOException, BundleException {
         InputStream is;
         String bundleLocation = bundleInfo.getLocation();
         LOGGER.debug("Checking " + bundleLocation);
@@ -575,6 +660,14 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
                 throw new BundleException("Manifest not present in the first entry of the zip " + bundleLocation);
             }
             String sn = m.getMainAttributes().getValue(Constants.BUNDLE_SYMBOLICNAME);
+            if (sn == null) {
+                throw new BundleException("Jar is not a bundle, no Bundle-SymbolicName " + bundleLocation);
+            }
+            // remove attributes from the symbolic name (like ;blueprint.graceperiod:=false suffix)
+            int attributeIndexSep = sn.indexOf(';');
+            if (attributeIndexSep != -1) {
+                sn = sn.substring(0, attributeIndexSep);
+            }
             String vStr = m.getMainAttributes().getValue(Constants.BUNDLE_VERSION);
             Version v = vStr == null ? Version.emptyVersion : Version.parseVersion(vStr);
             for (Bundle b : bundleContext.getBundles()) {
@@ -582,7 +675,10 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
                     vStr = (String) b.getHeaders().get(Constants.BUNDLE_VERSION);
                     Version bv = vStr == null ? Version.emptyVersion : Version.parseVersion(vStr);
                     if (v.equals(bv)) {
-                        LOGGER.debug("  found installed bundle: " + b);
+                        LOGGER.info("Found installed bundle: " + b);
+                        if (verbose) {
+                            System.out.println("Found installed bundle: " + b);
+                        }
                         state.bundles.add(b);
                         return b;
                     }
@@ -594,7 +690,10 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
                 is.close();
                 is = new BufferedInputStream(new URL(bundleLocation).openStream());
             }
-            LOGGER.debug("Installing bundle " + bundleLocation);
+            LOGGER.info("Installing bundle " + bundleLocation);
+            if (verbose) {
+                System.out.println("Installing bundle " + bundleLocation);
+            }
             Bundle b = getBundleContext().installBundle(bundleLocation, is);
             
             // Define the startLevel for the bundle when defined
@@ -610,6 +709,60 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
             is.close();
         }
     }
+    
+    public void installConfigurationFile(String fileLocation, String finalname, boolean override, boolean verbose) throws IOException {
+    	LOGGER.info("Checking configuration file " + fileLocation);
+        if (verbose) {
+            System.out.println("Checking configuration file " + fileLocation);
+        }
+    	
+    	String basePath = System.getProperty("karaf.base");
+    	
+    	if (finalname.indexOf("${") != -1) {
+    		//remove any placeholder or variable part, this is not valid.
+    		int marker = finalname.indexOf("}");
+    		finalname = finalname.substring(marker+1);
+    	}
+    	
+    	finalname = basePath + File.separator + finalname;
+    	
+    	File file = new File(finalname); 
+    	if (file.exists() && !override) {
+    		LOGGER.info("configFile already exist, don't override it");
+    		return;
+    	}
+    	if (!file.exists()) {
+    		File parentFile = file.getParentFile();
+    		if (parentFile != null)
+    			parentFile.mkdirs();
+    		file.createNewFile();
+    	}
+    	
+    	FileOutputStream fop = new FileOutputStream(file);
+        InputStream is = null;
+        try {
+            is = new BufferedInputStream(new URL(fileLocation).openStream());
+        
+            int bytesRead = 0;
+            byte[] buffer = new byte[1024];
+            
+            while ((bytesRead = is.read(buffer)) != -1) {
+                fop.write(buffer, 0, bytesRead);
+            }
+        } catch (RuntimeException e) {
+            LOGGER.error(e.getMessage());
+            throw e;
+        } catch (MalformedURLException e) {
+        	LOGGER.error(e.getMessage());
+            throw e;
+		} finally {
+			if (is != null)
+				is.close();
+			fop.flush();
+			fop.close();
+		}
+            
+    }
 
     public void uninstallFeature(String name) throws Exception {
         List<String> versions = new ArrayList<String>();
@@ -691,13 +844,25 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
             return null;
         } else {
             Feature feature = versions.get(version);
-            if (feature == null && FeatureImpl.DEFAULT_VERSION.equals(version)) {
-                Version latest = new Version(cleanupVersion(version));
-                for (String available : versions.keySet()) {
-                    Version availableVersion = new Version(cleanupVersion(available));
-                    if (availableVersion.compareTo(latest) > 0) {
-                        feature = versions.get(available);
-                        latest = availableVersion;
+            if (feature == null) {
+                if (FeatureImpl.DEFAULT_VERSION.equals(version)) {
+                    Version latest = new Version(cleanupVersion(version));
+                    for (String available : versions.keySet()) {
+                        Version availableVersion = new Version(cleanupVersion(available));
+                        if (availableVersion.compareTo(latest) > 0) {
+                            feature = versions.get(available);
+                            latest = availableVersion;
+                        }
+                    }
+                } else {
+                    Version latest = new Version(cleanupVersion(FeatureImpl.DEFAULT_VERSION));
+                    VersionRange versionRange = new VersionRange(version, true, true);
+                    for (String available : versions.keySet()) {
+                        Version availableVersion = new Version(cleanupVersion(available));
+                        if (availableVersion.compareTo(latest) > 0 && versionRange.contains(availableVersion)) {
+                            feature = versions.get(available);
+                            latest = availableVersion;
+                        }
                     }
                 }
             }
@@ -776,6 +941,7 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
                     String[] list = boot.split(",");
                     Set<Feature> features = new LinkedHashSet<Feature>();
                     for (String f : list) {
+                        f = f.trim();
                         if (f.length() > 0) {
                             String featureVersion = null;
 
@@ -871,19 +1037,16 @@ public class FeaturesServiceImpl implements FeaturesService, FrameworkListener {
         if (factoryPid == null) {
             filter = "(" + Constants.SERVICE_PID + "=" + pid + ")";
         } else {
-            filter = "(" + ConfigurationAdmin.SERVICE_FACTORYPID + "=" + factoryPid + ")";
+            String key = createConfigurationKey(pid, factoryPid);
+            filter = "(" + CONFIG_KEY + "=" + key + ")";
         }
         Configuration[] configurations = configurationAdmin.listConfigurations(filter);
-        if (configurations != null && configurations.length > 0)
-        {
+        if (configurations != null && configurations.length > 0) {
             return configurations[0];
         }
-        else
-        {
-            return null;
-        }
+        return null;
     }
-    
+
     protected void saveState() {
         try {
             File file = bundleContext.getDataFile("FeaturesServiceState.properties");
