@@ -30,7 +30,15 @@ import java.net.URLClassLoader;
 import java.security.AccessControlException;
 import java.security.Provider;
 import java.security.Security;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Properties;
+import java.util.Random;
+import java.util.StringTokenizer;
+import java.util.TreeMap;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.regex.Matcher;
@@ -118,6 +126,10 @@ public class Main {
      * holds the bundles data and cache for a Karaf instance.
      */
     public static final String ENV_KARAF_INSTANCES = "KARAF_INSTANCES";
+    /**
+     * The system property for holding the Karaf version.
+     */
+    public static final String PROP_KARAF_VERSION = "karaf.version";
 
     /**
      * Config property which identifies directories which contain bundles to be loaded by SMX
@@ -125,7 +137,8 @@ public class Main {
     public static final String BUNDLE_LOCATIONS = "bundle.locations";
 
     /**
-     * Config property that indicates we want to convert bundles locations to maven style urls
+     * Config property that indicates we want to convert bundles locations
+     * to Maven style URLs
      */
     public static final String PROPERTY_CONVERT_TO_MAVEN_URL = "karaf.maven.convert";
 
@@ -199,10 +212,10 @@ public class Main {
         karafData = Utils.getKarafDirectory(Main.PROP_KARAF_DATA, Main.ENV_KARAF_DATA, new File(karafBase, "data"), true, true);
         karafInstances = Utils.getKarafDirectory(Main.PROP_KARAF_INSTANCES, Main.ENV_KARAF_INSTANCES, new File(karafHome, "instances"), false, false);
 
-        //System.out.println("Karaf Home: "+main.karafHome.getPath());
-        //System.out.println("Karaf Base: "+main.karafBase.getPath());
-        //System.out.println("Karaf Data: "+main.karafData.getPath());
-
+        Package p = Package.getPackage("org.apache.karaf.main");
+        if (p != null && p.getImplementationVersion() != null) {
+            System.setProperty(PROP_KARAF_VERSION, p.getImplementationVersion());
+        }
         System.setProperty(PROP_KARAF_HOME, karafHome.getPath());
         System.setProperty(PROP_KARAF_BASE, karafBase.getPath());
         System.setProperty(PROP_KARAF_DATA, karafData.getPath());
@@ -221,7 +234,7 @@ public class Main {
         // Copy framework properties from the system properties.
         Main.copySystemProperties(configProps);
 
-        ClassLoader classLoader = updateClassLoader(configProps);
+        ClassLoader classLoader = createClassLoader(configProps);
 
         processSecurityProperties(configProps);
 
@@ -251,8 +264,11 @@ public class Main {
         }
         FrameworkFactory factory = (FrameworkFactory) classLoader.loadClass(factoryClass).newInstance();
         framework = factory.newFramework(new StringMap(configProps, false));
-        framework.start();
+        framework.init();
+        // Process properties
+        loadStartupProperties(configProps);
         processAutoProperties(framework.getBundleContext());
+        framework.start();
         // Start lock monitor
         new Thread() {
             public void run() {
@@ -262,6 +278,9 @@ public class Main {
     }
 
     public void awaitShutdown() throws Exception {
+        if (framework == null) {
+            return;
+        }
         while (true) {
             FrameworkEvent event = framework.waitForStop(0);
             if (event.getType() != FrameworkEvent.STOPPED_UPDATE) {
@@ -284,7 +303,7 @@ public class Main {
 
             // Stop the framework in case it's still active
             exiting = true;
-            if (framework.getState() == Bundle.ACTIVE) {
+            if (framework.getState() == Bundle.ACTIVE || framework.getState() == Bundle.STARTING) {
                 new Thread() {
                     public void run() {
                         try {
@@ -407,6 +426,7 @@ public class Main {
             try {
                 main.launch();
             } catch (Throwable ex) {
+                main.destroy();
                 main.setExitCode(-1);
                 System.err.println("Could not create framework: " + ex);
                 ex.printStackTrace();
@@ -440,15 +460,19 @@ public class Main {
         if (prop != null) {
             String[] providers = prop.split(",");
             for (String provider : providers) {
-                try {
-                    Security.addProvider((Provider) Class.forName(provider).newInstance());
-                } catch (Throwable t) {
-                    System.err.println("Unable to register security provider: " + t);
-                }
+                addProvider(provider);
             }
         }
     }
 
+    private static void addProvider(String provider) {
+        try {
+            Security.addProvider((Provider) Class.forName(provider).newInstance());
+        } catch (Throwable t) {
+            System.err.println("Unable to register security provider: " + t);
+        }
+    }
+
     private void updateInstancePid() {
         try {
             String instanceName = System.getProperty("karaf.name");
@@ -457,14 +481,15 @@ public class Main {
                 pid = pid.substring(0, pid.indexOf('@'));
             }
             
-            boolean isRoot = karafHome.equals(karafBase);
-            
             if (instanceName != null) {
                 String storage = System.getProperty("karaf.instances");
                 if (storage == null) {
                     throw new Exception("System property 'karaf.instances' is not set. \n" +
                         "This property needs to be set to the full path of the instance.properties file.");
                 }
+                boolean isRoot = karafHome.equals(karafBase)
+                        || karafBase.equals(new File(storage).getParentFile());
+
                 File storageFile = new File(storage);
                 File propertiesFile = new File(storageFile, "instance.properties");
                 Properties props = new Properties();
@@ -514,6 +539,8 @@ public class Main {
      * <p/>
      * Processes the auto-install and auto-start properties from the
      * specified configuration properties.
+     *
+     * @param context the system bundle context
      */
     private void processAutoProperties(BundleContext context) {
         // Check if we want to convert URLs to maven style
@@ -535,28 +562,31 @@ public class Main {
         }
         sl.setInitialBundleStartLevel(ibsl);
 
-        // The auto-install property specifies a space-delimited list of
-        // bundle URLs to be automatically installed into each new profile;
-        // the start level to which the bundles are assigned is specified by
-        // appending a ".n" to the auto-install property name, where "n" is
-        // the desired start level for the list of bundles.
-        autoInstall(PROPERTY_AUTO_INSTALL, context, sl, convertToMavenUrls, false);
+        // If we have a clean state, install everything
+        if (framework.getBundleContext().getBundles().length == 1) {
+            // The auto-install property specifies a space-delimited list of
+            // bundle URLs to be automatically installed into each new profile;
+            // the start level to which the bundles are assigned is specified by
+            // appending a ".n" to the auto-install property name, where "n" is
+            // the desired start level for the list of bundles.
+            autoInstall(PROPERTY_AUTO_INSTALL, context, sl, convertToMavenUrls, false);
 
-        // The auto-start property specifies a space-delimited list of
-        // bundle URLs to be automatically installed and started into each
-        // new profile; the start level to which the bundles are assigned
-        // is specified by appending a ".n" to the auto-start property name,
-        // where "n" is the desired start level for the list of bundles.
-        // The following code starts bundles in one pass, installing bundles
-        // for a given level, then starting them, then moving to the next level.
-        autoInstall(PROPERTY_AUTO_START, context, sl, convertToMavenUrls, true);
+            // The auto-start property specifies a space-delimited list of
+            // bundle URLs to be automatically installed and started into each
+            // new profile; the start level to which the bundles are assigned
+            // is specified by appending a ".n" to the auto-start property name,
+            // where "n" is the desired start level for the list of bundles.
+            // The following code starts bundles in one pass, installing bundles
+            // for a given level, then starting them, then moving to the next level.
+            autoInstall(PROPERTY_AUTO_START, context, sl, convertToMavenUrls, true);
+        }
     }
 
     private List<Bundle> autoInstall(String propertyPrefix, BundleContext context, StartLevel sl, boolean convertToMavenUrls, boolean start) {
         Map<Integer, String> autoStart = new TreeMap<Integer, String>();
         List<Bundle> bundles = new ArrayList<Bundle>();
-        for (Iterator i = configProps.keySet().iterator(); i.hasNext();) {
-            String key = (String) i.next();
+        for (Object o : configProps.keySet()) {
+            String key = (String) o;
             // Ignore all keys that are not the auto-start property.
             if (!key.startsWith(propertyPrefix)) {
                 continue;
@@ -568,8 +598,7 @@ public class Main {
             if (!key.equals(propertyPrefix)) {
                 try {
                     startLevel = Integer.parseInt(key.substring(key.lastIndexOf('.') + 1));
-                }
-                catch (NumberFormatException ex) {
+                } catch (NumberFormatException ex) {
                     System.err.println("Invalid property: " + key);
                 }
             }
@@ -618,7 +647,6 @@ public class Main {
         if (convertToMavenUrls) {
             String[] p = parts[1].split("/");
             if (p.length >= 4 && p[p.length-1].startsWith(p[p.length-3] + "-" + p[p.length-2])) {
-                String groupId = null;
                 String artifactId = p[p.length-3];
                 String version = p[p.length-2];
                 String classifier;
@@ -665,7 +693,7 @@ public class Main {
         if (st.countTokens() > 0) {
             String tokenList = "\" ";
             StringBuffer tokBuf = new StringBuffer(10);
-            String tok = null;
+            String tok;
             boolean inQuote = false;
             boolean tokStarted = false;
             boolean exit = false;
@@ -715,6 +743,8 @@ public class Main {
      * initializing the "<tt>felix.system.properties</tt>" system property to an
      * arbitrary URL.
      * </p>
+     *
+     * @param karafBase the karaf base folder
      */
     protected static void loadSystemProperties(File karafBase) {
         // The system properties file is either specified by a system
@@ -722,10 +752,10 @@ public class Main {
         // Try to load it from one of these places.
 
         // See if the property URL was specified as a property.
-        URL propURL = null;
+        URL propURL;
         try {
             File file = new File(new File(karafBase, "etc"), SYSTEM_PROPERTIES_FILE_NAME);
-            propURL = file.toURL();
+            propURL = file.toURI().toURL();
         }
         catch (MalformedURLException ex) {
             System.err.print("Main: " + ex);
@@ -760,7 +790,7 @@ public class Main {
         for (Enumeration e = props.propertyNames(); e.hasMoreElements();) {
             String name = (String) e.nextElement();
             String value = System.getProperty(name, props.getProperty(name));
-            System.setProperty(name, substVars(value, name, null, null));
+            System.setProperty(name, substVars(value, name, null, props));
         }
     }
 
@@ -780,27 +810,19 @@ public class Main {
      * </p>
      *
      * @return A <tt>Properties</tt> instance or <tt>null</tt> if there was an error.
-     * @throws Exception 
+     * @throws Exception if something wrong occurs
      */
     private Properties loadConfigProperties() throws Exception {
-        // The config properties file is either specified by a system
-        // property or it is in the conf/ directory of the Felix
-        // installation directory.  Try to load it from one of these
-        // places.
-
-        ArrayList<File> bundleDirs = new ArrayList<File>();
-
         // See if the property URL was specified as a property.
-        URL configPropURL = null;
-        URL startupPropURL = null;
+        URL configPropURL;
 
         try {
-            File file = new File(new File(karafBase, "etc"), CONFIG_PROPERTIES_FILE_NAME);
+            File etcFolder = new File(karafBase, "etc");
+            if (!etcFolder.exists()) {
+                throw new FileNotFoundException("etc folder not found: " + etcFolder.getAbsolutePath());
+            }
+            File file = new File(etcFolder, CONFIG_PROPERTIES_FILE_NAME);
             configPropURL = file.toURI().toURL();
-
-            file = new File(new File(karafBase, "etc"), STARTUP_PROPERTIES_FILE_NAME);
-            startupPropURL = file.toURI().toURL();
-
         }
         catch (MalformedURLException ex) {
             System.err.print("Main: " + ex);
@@ -809,23 +831,57 @@ public class Main {
 
 
         Properties configProps = loadPropertiesFile(configPropURL, false);
+
+        // Perform variable substitution for system properties.
+        for (Enumeration e = configProps.propertyNames(); e.hasMoreElements();) {
+            String name = (String) e.nextElement();
+            configProps.setProperty(name,
+                    substVars(configProps.getProperty(name), name, null, configProps));
+        }
+
+        return configProps;
+    }
+
+    private void loadStartupProperties(Properties configProps) throws Exception {
+        // The config properties file is either specified by a system
+        // property or it is in the conf/ directory of the Felix
+        // installation directory.  Try to load it from one of these
+        // places.
+
+        List<File> bundleDirs = new ArrayList<File>();
+
+        // See if the property URL was specified as a property.
+        URL startupPropURL;
+
+        File etcFolder = new File(karafBase, "etc");
+        if (!etcFolder.exists()) {
+            throw new FileNotFoundException("etc folder not found: " + etcFolder.getAbsolutePath());
+        }
+        File file = new File(etcFolder, STARTUP_PROPERTIES_FILE_NAME);
+        startupPropURL = file.toURI().toURL();
         Properties startupProps = loadPropertiesFile(startupPropURL, true);
 
         String defaultRepo = System.getProperty(DEFAULT_REPO, "system");
-
         if (karafBase.equals(karafHome)) {
-            bundleDirs.add(new File(karafHome, defaultRepo));
+            File systemRepo = new File(karafHome, defaultRepo);
+            if (!systemRepo.exists()) {
+                throw new FileNotFoundException("system repo not found: " + systemRepo.getAbsolutePath());
+            }
+            bundleDirs.add(systemRepo);
         } else {
-            bundleDirs.add(new File(karafBase, defaultRepo));
-            bundleDirs.add(new File(karafHome, defaultRepo));
+            File baseSystemRepo = new File(karafBase, defaultRepo);
+            File homeSystemRepo = new File(karafHome, defaultRepo);
+            if (!baseSystemRepo.exists() && !homeSystemRepo.exists()) {
+                throw new FileNotFoundException("system repos not found: " + baseSystemRepo.getAbsolutePath() + " " + homeSystemRepo.getAbsolutePath());
+            }
+            bundleDirs.add(baseSystemRepo);
+            bundleDirs.add(homeSystemRepo);
         }
-
         String locations = configProps.getProperty(BUNDLE_LOCATIONS);
-
         if (locations != null) {
             StringTokenizer st = new StringTokenizer(locations, "\" ", true);
             if (st.countTokens() > 0) {
-                String location = null;
+                String location;
                 do {
                     location = nextLocation(st);
                     if (location != null) {
@@ -848,17 +904,8 @@ public class Main {
             }
         }
 
-        // Perform variable substitution for system properties.
-        for (Enumeration e = configProps.propertyNames(); e.hasMoreElements();) {
-            String name = (String) e.nextElement();
-            configProps.setProperty(name,
-                    substVars(configProps.getProperty(name), name, null, configProps));
-        }
-
         // Mutate properties
         Main.processConfigurationProperties(configProps, startupProps, bundleDirs);
-
-        return configProps;
     }
 
     protected static Properties loadPropertiesFile(URL configPropURL, boolean failIfNotFound) throws Exception {
@@ -928,7 +975,7 @@ public class Main {
         }
     }
     
-    private ClassLoader updateClassLoader(Properties configProps) throws Exception {
+    private ClassLoader createClassLoader(Properties configProps) throws Exception {
     	String framework = configProps.getProperty(KARAF_FRAMEWORK);
         if (framework == null) {
             throw new IllegalArgumentException("Property " + KARAF_FRAMEWORK + " must be set in the etc/" + CONFIG_PROPERTIES_FILE_NAME + " configuration file");
@@ -956,21 +1003,23 @@ public class Main {
             }
         }
 
-        URLClassLoader classLoader = new URLClassLoader(urls.toArray(new URL[urls.size()]), Main.class.getClassLoader());
-        return classLoader;
+        return new URLClassLoader(urls.toArray(new URL[urls.size()]), Main.class.getClassLoader());
     }
 
     /**
      * Process properties to customize default felix behavior
      *
-     * @param startupProps
+     * @param configProps properties loaded from etc/config.properties
+     * @param startupProps properties loaded from etc/startup.properties
+     * @param bundleDirs location to load bundles from (usually system/)
      */
-    private static void processConfigurationProperties(Properties props, Properties startupProps, ArrayList<File> bundleDirs) {
+    private static void processConfigurationProperties(Properties configProps, Properties startupProps, List<File> bundleDirs) throws Exception {
         if (bundleDirs == null) {
             return;
         }
-        if ("all".equals(props.getProperty(PROPERTY_AUTO_START, "").trim())) {
-            props.remove(PROPERTY_AUTO_START);
+        boolean hasErrors = false;
+        if ("all".equals(configProps.getProperty(PROPERTY_AUTO_START, "").trim())) {
+            configProps.remove(PROPERTY_AUTO_START);
             ArrayList<File> jars = new ArrayList<File>();
 
             // We should start all the bundles in the system dir.
@@ -982,20 +1031,20 @@ public class Main {
 
             for (File jar : jars) {
                 try {
-                    sb.append("\"").append(jar.toURL().toString()).append("\" ");
+                    sb.append("\"").append(jar.toURI().toURL().toString()).append("\" ");
                 } catch (MalformedURLException e) {
                     System.err.print("Ignoring " + jar.toString() + " (" + e + ")");
                 }
             }
 
-            props.setProperty(PROPERTY_AUTO_START, sb.toString());
+            configProps.setProperty(PROPERTY_AUTO_START, sb.toString());
 
-        } else if (STARTUP_PROPERTIES_FILE_NAME.equals(props.getProperty(PROPERTY_AUTO_START, "").trim())) {
-            props.remove(PROPERTY_AUTO_START);
+        } else if (STARTUP_PROPERTIES_FILE_NAME.equals(configProps.getProperty(PROPERTY_AUTO_START, "").trim())) {
+            configProps.remove(PROPERTY_AUTO_START);
             // We should start the bundles in the startup.properties file.
             HashMap<Integer, StringBuffer> levels = new HashMap<Integer, StringBuffer>();
-            for (Iterator iterator = startupProps.keySet().iterator(); iterator.hasNext();) {
-                String name = (String) iterator.next();
+            for (Object o : startupProps.keySet()) {
+                String name = (String) o;
                 File file = findFile(bundleDirs, name);
 
                 if (file != null) {
@@ -1012,23 +1061,26 @@ public class Main {
                         levels.put(level, sb);
                     }
                     try {
-                        sb.append("\"").append(file.toURL().toString()).append("|").append(name).append("\" ");
+                        sb.append("\"").append(file.toURI().toURL().toString()).append("|").append(name).append("\" ");
                     } catch (MalformedURLException e) {
                         System.err.print("Ignoring " + file.toString() + " (" + e + ")");
                     }
                 } else {
                     System.err.println("Bundle listed in " + STARTUP_PROPERTIES_FILE_NAME + " configuration not found: " + name);
+                    hasErrors = true;
                 }
             }
 
             for (Map.Entry<Integer, StringBuffer> entry : levels.entrySet()) {
-                props.setProperty(PROPERTY_AUTO_START + "." + entry.getKey(), entry.getValue().toString());
+                configProps.setProperty(PROPERTY_AUTO_START + "." + entry.getKey(), entry.getValue().toString());
             }
         }
-
+        if (hasErrors) {
+            throw new Exception("Aborting due to missing startup bundles");
+        }
     }
 
-    private static File findFile(ArrayList<File> bundleDirs, String name) {
+    private static File findFile(List<File> bundleDirs, String name) {
         for (File bundleDir : bundleDirs) {
             File file = findFile(bundleDir, name);
             if (file != null) {
@@ -1181,9 +1233,11 @@ public class Main {
         return val;
     }
 
-    /* (non-Javadoc)
-      * @see org.apache.karaf.main.MainService#getArgs()
-      */
+    /**
+     * Retrieve the arguments used when launching Karaf
+     *
+     * @return the arguments of the main karaf process
+     */
     public String[] getArgs() {
         return args;
     }
@@ -1203,33 +1257,7 @@ public class Main {
     public void lock(Properties props) {
         try {
             if (Boolean.parseBoolean(props.getProperty(PROPERTY_USE_LOCK, "true"))) {
-                String clz = props.getProperty(PROPERTY_LOCK_CLASS, PROPERTY_LOCK_CLASS_DEFAULT);
-                lock = (Lock) Class.forName(clz).getConstructor(Properties.class).newInstance(props);
-                boolean lockLogged = false;
-                setStartLevel(lockStartLevel);
-                while (!exiting) {
-                    if (lock.lock()) {
-                        if (lockLogged) {
-                            LOG.info("Lock acquired.");
-                        }
-                        setupShutdown(props);
-                        setStartLevel(defaultStartLevel);
-                        for (;;) {
-                            if (!lock.isAlive()) {
-                                break;
-                            }
-                            Thread.sleep(lockDelay);
-                        }
-                        if (framework.getState() == Bundle.ACTIVE && !exiting) {
-                            LOG.info("Lost the lock, stopping this instance ...");
-                            setStartLevel(lockStartLevel);
-                        }
-                    } else if (!lockLogged) {
-                        LOG.info("Waiting for the lock ...");
-                        lockLogged = true;
-                    }
-                    Thread.sleep(lockDelay);
-                } 
+                doLock(props);
             } else {
                 setStartLevel(defaultStartLevel);
             }
@@ -1238,6 +1266,36 @@ public class Main {
         }
     }
 
+    private void doLock(Properties props) throws Exception {
+        String clz = props.getProperty(PROPERTY_LOCK_CLASS, PROPERTY_LOCK_CLASS_DEFAULT);
+        lock = (Lock) Class.forName(clz).getConstructor(Properties.class).newInstance(props);
+        boolean lockLogged = false;
+        setStartLevel(lockStartLevel);
+        while (!exiting) {
+            if (lock.lock()) {
+                if (lockLogged) {
+                    LOG.info("Lock acquired.");
+                }
+                setupShutdown(props);
+                setStartLevel(defaultStartLevel);
+                for (;;) {
+                    if (!lock.isAlive()) {
+                        break;
+                    }
+                    Thread.sleep(lockDelay);
+                }
+                if (framework.getState() == Bundle.ACTIVE && !exiting) {
+                    LOG.info("Lost the lock, stopping this instance ...");
+                    setStartLevel(lockStartLevel);
+                }
+            } else if (!lockLogged) {
+                LOG.info("Waiting for the lock ...");
+                lockLogged = true;
+            }
+            Thread.sleep(lockDelay);
+        }
+    }
+
     public void unlock() throws Exception {
         if (lock != null) {
             lock.release();
@@ -1256,6 +1314,32 @@ public class Main {
     private ServerSocket shutdownSocket;
 
     protected void setupShutdown(Properties props) {
+        writePid(props);
+        try {
+            int port = Integer.parseInt(props.getProperty(KARAF_SHUTDOWN_PORT, "0"));
+            String host = props.getProperty(KARAF_SHUTDOWN_HOST, "localhost");
+            String portFile = props.getProperty(KARAF_SHUTDOWN_PORT_FILE);
+            final String shutdown = props.getProperty(KARAF_SHUTDOWN_COMMAND, DEFAULT_SHUTDOWN_COMMAND);
+            if (port >= 0) {
+                shutdownSocket = new ServerSocket(port, 1, InetAddress.getByName(host));
+                if (port == 0) {
+                    port = shutdownSocket.getLocalPort();
+                }
+                if (portFile != null) {
+                    Writer w = new OutputStreamWriter(new FileOutputStream(portFile));
+                    w.write(Integer.toString(port));
+                    w.close();
+                }
+                Thread thread = new ShutdownSocketThread(shutdown);
+                thread.setDaemon(true);
+                thread.start();
+            }
+        } catch (Exception e) {
+            e.printStackTrace();
+        }
+    }
+
+    private void writePid(Properties props) {
         try {
             String pidFile = props.getProperty(KARAF_SHUTDOWN_PID_FILE);
             if (pidFile != null) {
@@ -1273,99 +1357,86 @@ public class Main {
         } catch (Exception e) {
             e.printStackTrace();
         }
-        try {
-            int port = Integer.parseInt(props.getProperty(KARAF_SHUTDOWN_PORT, "0"));
-            String host = props.getProperty(KARAF_SHUTDOWN_HOST, "localhost");
-            String portFile = props.getProperty(KARAF_SHUTDOWN_PORT_FILE);
-            final String shutdown = props.getProperty(KARAF_SHUTDOWN_COMMAND, DEFAULT_SHUTDOWN_COMMAND);
-            if (port >= 0) {
-                shutdownSocket = new ServerSocket(port, 1, InetAddress.getByName(host));
-                if (port == 0) {
-                    port = shutdownSocket.getLocalPort();
-                }
-                if (portFile != null) {
-                    Writer w = new OutputStreamWriter(new FileOutputStream(portFile));
-                    w.write(Integer.toString(port));
-                    w.close();
-                }
-                Thread thread = new Thread() {
-                    public void run() {
-                        try {
-                            while (true) {
-                                // Wait for the next connection
-                                Socket socket = null;
-                                InputStream stream = null;
-                                try {
-                                    socket = shutdownSocket.accept();
-                                    socket.setSoTimeout(10 * 1000);  // Ten seconds
-                                    stream = socket.getInputStream();
-                                } catch (AccessControlException ace) {
-                                    LOG.log(Level.WARNING, "Karaf shutdown socket: security exception: "
-                                                       + ace.getMessage(), ace);
-                                    continue;
-                                } catch (IOException e) {
-                                    LOG.log(Level.SEVERE, "Karaf shutdown socket: accept: ", e);
-                                    System.exit(1);
-                                }
-
-                                // Read a set of characters from the socket
-                                StringBuilder command = new StringBuilder();
-                                int expected = 1024; // Cut off to avoid DoS attack
-                                while (expected < shutdown.length()) {
-                                    if (random == null) {
-                                        random = new Random();
-                                    }
-                                    expected += (random.nextInt() % 1024);
-                                }
-                                while (expected > 0) {
-                                    int ch = -1;
-                                    try {
-                                        ch = stream.read();
-                                    } catch (IOException e) {
-                                        LOG.log(Level.WARNING, "Karaf shutdown socket:  read: ", e);
-                                        ch = -1;
-                                    }
-                                    if (ch < 32) {  // Control character or EOF terminates loop
-                                        break;
-                                    }
-                                    command.append((char) ch);
-                                    expected--;
-                                }
-
-                                // Close the socket now that we are done with it
-                                try {
-                                    socket.close();
-                                } catch (IOException e) {
-                                    // Ignore
-                                }
-
-                                // Match against our command string
-                                boolean match = command.toString().equals(shutdown);
-                                if (match) {
-                                    LOG.log(Level.INFO, "Karaf shutdown socket: received shutdown command. Stopping framework...");
-                                    framework.stop();
-                                    break;
-                                } else {
-                                    LOG.log(Level.WARNING, "Karaf shutdown socket:  Invalid command '" +
-                                                       command.toString() + "' received");
-                                }
-                            }
-                        } catch (Exception e) {
-                            e.printStackTrace();
-                        } finally {
-                            try {
-                                shutdownSocket.close();
-                            } catch (IOException e) {
-                            }
-                        }
-                    }
-                };
-                thread.setDaemon(true);
-                thread.start();
-            }
-        } catch (Exception e) {
-            e.printStackTrace();
-        }
     }
 
+    private class ShutdownSocketThread extends Thread {
+
+        private final String shutdown;
+
+        public ShutdownSocketThread(String shutdown) {
+            this.shutdown = shutdown;
+        }
+
+        public void run() {
+            try {
+                while (true) {
+                    // Wait for the next connection
+                    Socket socket = null;
+                    InputStream stream = null;
+                    try {
+                        socket = shutdownSocket.accept();
+                        socket.setSoTimeout(10 * 1000);  // Ten seconds
+                        stream = socket.getInputStream();
+                    } catch (AccessControlException ace) {
+                        LOG.log(Level.WARNING, "Karaf shutdown socket: security exception: "
+                                           + ace.getMessage(), ace);
+                        continue;
+                    } catch (IOException e) {
+                        LOG.log(Level.SEVERE, "Karaf shutdown socket: accept: ", e);
+                        System.exit(1);
+                    }
+
+                    // Read a set of characters from the socket
+                    StringBuilder command = new StringBuilder();
+                    int expected = 1024; // Cut off to avoid DoS attack
+                    while (expected < shutdown.length()) {
+                        if (random == null) {
+                            random = new Random();
+                        }
+                        expected += (random.nextInt() % 1024);
+                    }
+                    while (expected > 0) {
+                        int ch;
+                        try {
+                            ch = stream.read();
+                        } catch (IOException e) {
+                            LOG.log(Level.WARNING, "Karaf shutdown socket:  read: ", e);
+                            ch = -1;
+                        }
+                        if (ch < 32) {  // Control character or EOF terminates loop
+                            break;
+                        }
+                        command.append((char) ch);
+                        expected--;
+                    }
+
+                    // Close the socket now that we are done with it
+                    try {
+                        socket.close();
+                    } catch (IOException e) {
+                        // Ignore
+                    }
+
+                    // Match against our command string
+                    boolean match = command.toString().equals(shutdown);
+                    if (match) {
+                        LOG.log(Level.INFO, "Karaf shutdown socket: received shutdown command. Stopping framework...");
+                        framework.stop();
+                        break;
+                    } else {
+                        LOG.log(Level.WARNING, "Karaf shutdown socket:  Invalid command '" +
+                                           command.toString() + "' received");
+                    }
+                }
+            } catch (Exception e) {
+                e.printStackTrace();
+            } finally {
+                try {
+                    shutdownSocket.close();
+                } catch (IOException e) {
+                    // Ignore
+                }
+            }
+        }
+    }
 }
