@@ -39,9 +39,9 @@ import org.apache.felix.gogo.commands.Argument;
 import org.apache.felix.gogo.commands.Command;
 import org.apache.felix.gogo.commands.converter.DefaultConverter;
 import org.apache.felix.gogo.commands.converter.GenericType;
+import org.apache.felix.service.command.CommandSession;
 import org.apache.karaf.shell.console.NameScoping;
 import org.fusesource.jansi.Ansi;
-import org.osgi.service.command.CommandSession;
 
 public class DefaultActionPreparator implements ActionPreparator {
 
@@ -71,6 +71,11 @@ public class DefaultActionPreparator implements ActionPreparator {
             return false;
         }
 
+        public String valueToShowInHelp()
+        {
+            return Option.DEFAULT_STRING;
+        }
+
         public Class<? extends Annotation> annotationType()
         {
             return Option.class;
@@ -110,6 +115,9 @@ public class DefaultActionPreparator implements ActionPreparator {
                             public boolean multiValued() {
                                 return delegate.multiValued();
                             }
+                            public String valueToShowInHelp() {
+                                return delegate.valueToShowInHelp();
+                            }
                             public Class<? extends Annotation> annotationType() {
                                 return delegate.annotationType();
                             }
@@ -442,20 +450,20 @@ public class DefaultActionPreparator implements ActionPreparator {
             {
                 out.print("        ");
                 out.println(Ansi.ansi().a(Ansi.Attribute.INTENSITY_BOLD).a(argument.name()).a(Ansi.Attribute.RESET));
-                printFormatted("                ", argument.description(), term != null ? term.getTerminalWidth() : 80, out);
+                printFormatted("                ", argument.description(), term != null ? term.getWidth() : 80, out);
                 if (!argument.required()) {
-                    try {
-                        argsMap.get(argument).setAccessible(true);
-                        Object o = argsMap.get(argument).get(action);
-                        if (o != null
-                                && (!(o instanceof Boolean) || ((Boolean) o))
-                                && (!(o instanceof Number) || ((Number) o).doubleValue() != 0.0)) {
-                            out.print("                    (defaults to ");
-                            out.print(o.toString());
-                            out.println(")");
+                    if (argument.valueToShowInHelp() != null && argument.valueToShowInHelp().length() != 0) {
+                         try {
+                            if (Argument.DEFAULT_STRING.equals(argument.valueToShowInHelp())) {
+                                argsMap.get(argument).setAccessible(true);
+                                Object o = argsMap.get(argument).get(action);
+                                printObjectDefaultsTo(out, o);
+                            } else {
+                                printDefaultsTo(out, argument.valueToShowInHelp());
+                            }
+                        } catch (Throwable t) {
+                            // Ignore
                         }
-                    } catch (Throwable t) {
-                        // Ignore
                     }
                 }
             }
@@ -473,19 +481,19 @@ public class DefaultActionPreparator implements ActionPreparator {
                 }
                 out.print("        ");
                 out.println(Ansi.ansi().a(Ansi.Attribute.INTENSITY_BOLD).a(opt).a(Ansi.Attribute.RESET));
-                printFormatted("                ", option.description(), term != null ? term.getTerminalWidth() : 80, out);
-                try {
-                    optionsMap.get(option).setAccessible(true);
-                    Object o = optionsMap.get(option).get(action);
-                    if (o != null
-                            && (!(o instanceof Boolean) || ((Boolean) o))
-                            && (!(o instanceof Number) || ((Number) o).doubleValue() != 0.0)) {
-                        out.print("                (defaults to ");
-                        out.print(o.toString());
-                        out.println(")");
+                printFormatted("                ", option.description(), term != null ? term.getWidth() : 80, out);
+                if (option.valueToShowInHelp() != null && option.valueToShowInHelp().length() != 0) {
+                    try {
+                        if(Option.DEFAULT_STRING.equals(option.valueToShowInHelp())) {
+                            optionsMap.get(option).setAccessible(true);
+                            Object o = optionsMap.get(option).get(action);
+                            printObjectDefaultsTo(out, o);
+                        } else {
+                            printDefaultsTo(out, option.valueToShowInHelp());
+                        }
+                    } catch (Throwable t) {
+                        // Ignore
                     }
-                } catch (Throwable t) {
-                    // Ignore
                 }
             }
             out.println();
@@ -493,11 +501,25 @@ public class DefaultActionPreparator implements ActionPreparator {
         if (command.detailedDescription().length() > 0) {
             out.println(Ansi.ansi().a(Ansi.Attribute.INTENSITY_BOLD).a("DETAILS").a(Ansi.Attribute.RESET));
             String desc = loadDescription(action.getClass(), command.detailedDescription());
-            printFormatted("        ", desc, term != null ? term.getTerminalWidth() : 80, out);
+            printFormatted("        ", desc, term != null ? term.getWidth() : 80, out);
         }
     }
 
-    private String loadDescription(Class clazz, String desc) {
+    private void printObjectDefaultsTo(PrintStream out, Object o) {
+        if (o != null
+                && (!(o instanceof Boolean) || ((Boolean) o))
+                && (!(o instanceof Number) || ((Number) o).doubleValue() != 0.0)) {
+            printDefaultsTo(out, o.toString());
+        }
+    }
+
+    private void printDefaultsTo(PrintStream out, String value) {
+        out.print("                (defaults to ");
+        out.print(value);
+        out.println(")");
+    }
+
+    protected String loadDescription(Class clazz, String desc) {
         if (desc.startsWith("classpath:")) {
             InputStream is = clazz.getResourceAsStream(desc.substring("classpath:".length()));
             if (is == null) {
